#include <driver.h>

#include "serialInterface.h"

#include <klibc/assert.h>

// TODO: Remove for driver(In/Out)Port(8/16/32)();
#include <arch/i386/io.h>

#define REGISTER_OFFSET_RECV_BUFFER							(0)
#define REGISTER_OFFSET_SEND_BUFFER							(0)
#define REGISTER_OFFSET_INTERRUPT_ENABLE					(1)
#define REGISTER_OFFSET_INTERRUPT_IDENT					(2)
#define REGISTER_OFFSET_DATA_FORMAT							(3)
#define REGISTER_OFFSET_MODEM_CONTROL						(4)
#define REGISTER_OFFSET_LINE_STATUS							(5)
#define REGISTER_OFFSET_MODEM_STATUS						(6)
#define REGISTER_OFFSET_SCRATCH_PAD							(7)
#define REGISTER_OFFSET_DIVISOR_LATCH_LSB					(0)
#define REGISTER_OFFSET_DIVISOR_LATCH_MSB					(1)

/** DATA FORMAT REGISTER MASKS */
#define REGISTER_MASK_DATA_FORMAT_DLAB						(0x80)
#define REGISTER_MASK_DATA_FORMAT_NO_DLAB_ACCESS		(0x7F)
#define REGISTER_MASK_DATA_FORMAT_BRK						(0x40)
#define REGISTER_MASK_DATA_FORMAT_PARITY					(0x38)
#define REGISTER_MASK_DATA_FORMAT_STOP						(0x04)
#define REGISTER_MASK_DATA_FORMAT_DBIT						(0x03)

/** DATA FORMAT REGISTER VALUES */
#define REGISTER_VALUE_DATA_FORMAT_DLAB_ACCESS			(0x80)
#define REGISTER_VALUE_DATA_FORMAT_BREAK_ON				(0x40)
#define REGISTER_VALUE_DATA_FORMAT_PARITY_NONE			(0x00)
#define REGISTER_VALUE_DATA_FORMAT_PARITY_ODD			(0x08)
#define REGISTER_VALUE_DATA_FORMAT_PARITY_EVEN			(0x18)
#define REGISTER_VALUE_DATA_FORMAT_PARITY_MARK			(0x28)
#define REGISTER_VALUE_DATA_FORMAT_PARITY_SPACE			(0x38)
#define REGISTER_VALUE_DATA_FORMAT_STOP_2					(0x04)
#define REGISTER_VALUE_DATA_FORMAT_STOP_1					(0x00)
#define REGISTER_VALUE_DATA_FORMAT_DBIT_5					(0X00)
#define REGISTER_VALUE_DATA_FORMAT_DBIT_6					(0x01)
#define REGISTER_VALUE_DATA_FORMAT_DBIT_7					(0x02)
#define REGISTER_VALUE_DATA_FORMAT_DBIT_8					(0x03)

/** TODO: Need to figure out how to handle private context data generally, and also
* in this driver.  Private context data should probably be attached to a DriverContext, not
* a driver.  As such, these functions should accept DriverContext params, rather then Driver
* params (A driver can be returned from a DriverContext anyway!).
*
* In the case of this serial driver, however, it may almost make more sense to simply have a
* global array of 4 _SerialContext structures.
*/
typedef struct _SerialContext
{
   uint8 port;
   uint8 dataBits;
   uint8 stopBits;
   uint8 parity;
   uint32 baud;
}             *SerialContext;

ErrorCode _consoleCharListener(int c);
void _serialPutChar(uint8 port, int c);
uint8 _serialGetDataBits(uint8 port);
uint8 _serialGetStopBits(uint8 port);
uint8 _serialGetParity(uint8 port);
uint32 _serialGetBaud(uint8 port);
void _serialAccessDivisorLatch(Boolean accessable);

uint16 _baseRegister[4] = { 0X3f8, 0X2f8, 0x3e8, 0x2e8 };

ErrorCode serialDetect(Driver drv)
{
   static int times = 0;

   printf("serialDetect() called %d times\n", ++times);

   /* As a result of a strange object linking bug, an unlinked symbol call resulted
    * in calling this serialDetect() function instead.  The below code is a remanent
    * of that, and left as a reminder, for the time being
    */
   if(times == 2)
      assert(0);

   return ErrorNoError;
}

ErrorCode serialFinal(Driver drv)
{
   printf("serialFinal()\n");
   return ErrorNoError;
}

DriverVariable driverSerialRoot(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;
   SerialContext ctx;

   /** TODO: should ports be 0 or 1 based? */
   printf("driverSerialRoot() -> serial port %d\n", vars[0].integer);

   /* this is our entry into the driver... create a context */
   ctx = (SerialContext) malloc(sizeof(struct _SerialContext));
   ctx->port = vars[0].integer;
   ctx->dataBits = _serialGetDataBits(ctx->port);
   ctx->stopBits = _serialGetStopBits(ctx->port);
   ctx->parity = _serialGetParity(ctx->port);
   ctx->baud = _serialGetBaud(ctx->port);

   printf("driver settings %d,%d%c%d\n", ctx->baud, (int)ctx->dataBits, (char)ctx->parity,
          (int)ctx->stopBits);

   // TODO: set context

   return ret;
}

DriverVariable driverSerialGet(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;
   uint8 port = 0;

   /** TODO: get context! */

   printf("driverSerialGet()\n");

   ret = _serialGetChar(port);

   return ret;
}

DriverVariable driverSerialPut(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;
   uint8 port = 0;

   /** TODO: get context! */

   printf("driverSerialPut('%c') on serial %d\n", vars[0].character, port);

   _serialPutChar(port, vars[0].character);

   return ret;
}

DriverVariable driverSerialRead(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;
   uint32 bufferSize, i;
   char *buffer;
   uint8 port = 0;

   /** TODO: get context! */

   printf("driverSerialRead()\n");

   bufferSize = vars[0].integer;
   buffer = (char *)malloc(bufferSize);
   if(buffer)
   {
      for(i = 0; i < bufferSize; i++)
      {
         buffer[i] = _serialGetChar(port);
      }
   }

   return ret;
}

DriverVariable driverSerialWrite(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;
   uint32 bufferSize, i;
   char *buffer;
   uint8 port = 0;

   /** TODO: get context! */

   printf("driverSerialWrite()\n");

   bufferSize = vars[0].integer;
   buffer = (char *)vars[1].pointer;
   if(bufferSize && buffer)
   {
      for(i = 0; i < bufferSize; i++)
      {
         _serialPutChar(port, buffer[i]);
      }
   }

   return ret;
}

DriverVariable driverSerialGetBaud(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialGetBaud()\n");

   return ret;
}

DriverVariable driverSerialSetBaud(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialSetBaud()\n");

   return ret;
}

DriverVariable driverSerialGetDataBits(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialGetDataBits()\n");

   return ret;
}

DriverVariable driverSerialSetDataBits(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialSetDataBits()\n");

   return ret;
}

DriverVariable driverSerialGetStopBits(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialGetStopBits()\n");

   return ret;
}

DriverVariable driverSerialSetStopBits(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialSetStopBits()\n");

   return ret;
}

DriverVariable driverSerialGetParity(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialGetParity()\n");

   return ret;
}

DriverVariable driverSerialSetParity(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialSetParity()\n");

   return ret;
}

DriverVariable driverSerialTraceEnable(Driver drv, DriverVariable vars[])
{
   DriverVariable ret;

   printf("driverSerialTraceEnable(%d)\n", vars[0].boolean);

   if(vars[0].boolean)
   {
      consoleEnableCharacterListener(_consoleCharListener);
   }
   else
   {
      consoleDisableCharacterListener(_consoleCharListener);
   }

   return ret;
}

ErrorCode _consoleCharListener(int c)
{
   /** TODO: save off character listener port in driverSerialTraceEnable */
   _serialPutChar(0, c);
}

void _serialPutChar(uint8 port, int c)
{
   outportb(_baseRegister[port] + REGISTER_OFFSET_SEND_BUFFER, c);
}

uint8 _serialGetDataBits(uint8 port)
{
   uint8 value, dataBits;

   //driverInPort8(_baseRegister[port] + REGISTER_OFFSET_LINE_CONTROL, &value);
   value = inportb(_baseRegister[port] + REGISTER_OFFSET_DATA_FORMAT);

   switch (value & REGISTER_MASK_DATA_FORMAT_DBIT)
   {
   case REGISTER_VALUE_DATA_FORMAT_DBIT_5:
      dataBits = 5;
      break;
   case REGISTER_VALUE_DATA_FORMAT_DBIT_6:
      dataBits = 6;
      break;
   case REGISTER_VALUE_DATA_FORMAT_DBIT_7:
      dataBits = 7;
      break;
   case REGISTER_VALUE_DATA_FORMAT_DBIT_8:
      dataBits = 8;
      break;
   default:
      dataBits = 0;
   }

   return dataBits;
}

uint8 _serialGetStopBits(uint8 port)
{
   uint8 value, stopBits;

   //driverInPort8(_baseRegister[port] + REGISTER_OFFSET_LINE_CONTROL, &value);
   value = inportb(_baseRegister[port] + REGISTER_OFFSET_DATA_FORMAT);

   if(value & REGISTER_MASK_DATA_FORMAT_STOP)
   {
      stopBits = 2;
   }
   else
   {
      stopBits = 1;
   }

   return stopBits;
}

uint8 _serialGetParity(uint8 port)
{
   uint8 value;
   char parity;

   //driverInPort8(_baseRegister[port] + REGISTER_OFFSET_LINE_CONTROL, &value);
   value = inportb(_baseRegister[port] + REGISTER_OFFSET_DATA_FORMAT);

   switch (value & REGISTER_MASK_DATA_FORMAT_PARITY)
   {
   case REGISTER_VALUE_DATA_FORMAT_PARITY_NONE:
      parity = 'n';
      break;
   case REGISTER_VALUE_DATA_FORMAT_PARITY_ODD:
      parity = 'o';
      break;
   case REGISTER_VALUE_DATA_FORMAT_PARITY_EVEN:
      parity = 'e';
      break;
   case REGISTER_VALUE_DATA_FORMAT_PARITY_MARK:
      parity = 'm';
      break;
   case REGISTER_VALUE_DATA_FORMAT_PARITY_SPACE:
      parity = 's';
      break;
   default:
      parity = '0';
   }

   return (uint8)parity;
}

uint32 _serialGetBaud(uint8 port)
{
   uint8 divisorHigh, divisorLow;
   uint16 baudDivisor;
   uint32 baudRate;

   /* give access to the divisor latch */
   _serialAccessDivisorLatch(True);

   /* read the divisor halfs, and assemble them together */
   divisorLow = inportb(_baseRegister[port] + REGISTER_OFFSET_DIVISOR_LATCH_LSB);
   divisorHigh = inportb(_baseRegister[port] + REGISTER_OFFSET_DIVISOR_LATCH_MSB);
   baudDivisor = divisorLow + (((uint16)divisorHigh) << 8);

   if(baudDivisor == 0)
   {
      baudRate = 0;
   }
   else
   {
      baudRate = 115200 / baudDivisor;
   }

   /* remove access to divisor latch */
   _serialAccessDivisorLatch(False);

   return baudRate;
}

void _serialAccessDivisorLatch(Boolean accessable)
{
   uint8 dataFormat;

   dataFormat = inportb(_baseRegister[port] + REGISTER_OFFSET_DATA_FORMAT);
   if(accessable)
   {
      dataFormat |= REGISTER_VALUE_DATA_FORMAT_DLAB_ACCESS;
   }
   else
   {
      dataFormat &= REGISTER_MASK_DATA_FORMAT_NO_DLAB_ACCESS;
   }
   outportb(_baseRegister[port] + REGISTER_OFFSET_DATA_FORMAT, dataFormat);
}
