/* ndk - [ messageQueue.h ]
 *
 * Basic message queue class for IPC
 *
 * (c)2004 dcipher / neuraldk
 *           www.neuraldk.org
 */

/**
* @defgroup MessageQueue messageQueue.h: Basic message queue class
* @ingroup core
*
* This file defines a interface for working with message queues
*/
/** @{ */

#ifndef __ndk_messageQueue_h__
#define __ndk_messageQueue_h__

#include <errorCodes.h>
#include <types.h>

typedef struct _MessageQueue *MessageQueue;
typedef struct _Message *Message;

typedef ErrorCode(*MessageDestroyFunction) (Message msg);

ErrorCode messageQueueCreate(MessageQueue *msgq, uint32 size);
ErrorCode messageQueueSend(MessageQueue mq, Pointer message);
ErrorCode messageQueuePeek(MessageQueue mq, Pointer *message);
ErrorCode messageQueueReceive(MessageQueue mq, Timeout to, Pointer *message);
ErrorCode messageQueueDestroy(MessageQueue *mq);

/** TODO: implement below... */
ErrorCode messageCreate(Message * msg, MessageDestroyFunction destroy, MessageQueue responseQueue,
                        uint32 code, uint32 numArgs, ...);
ErrorCode messageGetArgument(Message msg, uint32 num, Pointer *arg);
ErrorCode messageGetArgumentCount(Message msg, uint32 *total);
ErrorCode messageGetArgumentList(Message msg, Pointer *top, uint32 *total);
ErrorCode messageGetCode(Message msg, uint32 *code);
ErrorCode messageRespond(Message source, Message response);
ErrorCode messageDestroy(Message * msg);

#endif

/** @} */
