/* ndk - [ objectKernel.h ]
 *
 * Implements an 'Object' class for the kernel
 *
 * (c)2004 dcipher / neuraldk
 *           www.neuraldk.org
 */

#include "objectKernel.h"
#include "object.h"

// for the export declarations
#include <console.h>
#include <memory.h>
#include <mutex.h>
#include <timer.h>
#include <messageQueue.h>
#include <driver.h>
#include <klibc/string.h>
#include <klibc/printf.h>
#include <klibc/assert.h>
#include <klibc/malloc.h>
// TODO: remove i386 dep!!!
#include <arch/i386/io.h>
#include <arch/i386/memory.h>

#define addExport(sym)	\
	{ #sym, (void*)sym }

#define addImport(sym)	\
	{ #sym, (void*)&sym }

Object objectKernel;

typedef struct _SymbolMapping
{
   char *name;
   void *location;
} SymbolMapping;

// TODO: get rid of these externs!
extern void *textEnd;
extern void *dataEnd;
extern void *bssEnd;

// TODO: find a better method to define these imports!
void (*splashScreen) (void *argument) = 0;

static const SymbolMapping symExports[] = {
   addExport(_assert),
   addExport(consoleClear),
   addExport(consoleEnableCharacterListener),
   addExport(consoleDisableCharacterListener),
   addExport(consoleLockScrollWindow),

   /** TODO: alphabetize these driver funcs, and ensure all that are needed are in the export list */
   addExport(driverItemCreateFunction),
   addExport(driverItemCreateParameter),
   addExport(driverItemCreateNamespace),
   addExport(driverItemAdd),
   addExport(driverParamCreate),
   addExport(driverParamDestroy),
   addExport(driverSetRoot),
   addExport(driverSetMetaData),
   addExport(driverGetMessageQueue),
   addExport(driverReserveIrq),
   addExport(driverReservePort),
   addExport(driverReserveMemory),
   addExport(driverReserveIrqRange),
   addExport(driverReservePortRange),
   addExport(driverReserveMemoryRange),

   addExport(printf),
   addExport(inportb),
   addExport(malloc),
   addExport(memcpy),
   // TODO: remove memory*
   //addExport(memoryAllocPage),
   //addExport(memoryFreePage),
   // malloc?
   addExport(memset),
   addExport(messageQueueCreate),
   addExport(messageQueueDestroy),
   addExport(messageQueuePeek),
   addExport(messageQueueReceive),
   addExport(messageQueueSend),
   addExport(mutexCreate),
   addExport(mutexDestroy),
   addExport(mutexLock),
   addExport(mutexUnlock),
   addExport(outportb),
   addExport(outportw),
   addExport(printf),
   //{ "multibootInfo"  , (long)multibootInfo  , 0 },
   addExport(strcmp),
   addExport(strncmp),
   addExport(timerGetTicks),
   {(char *)NULL, NULL}
};

static const SymbolMapping symImports[] = {
   addImport(splashScreen),
   {(char *)NULL, NULL}
};

ErrorCode objectKernelInit(void)
{
   uint32 i = 0;
   ObjectSection text, data, bss;
   ObjectRelocation reloc;
   ObjectSymbol symbol;

   objectCreateEmpty(&objectKernel);

   objectKernel->name = "ndk";
   objectKernel->base = (uint8 *)0x00000000;
   objectKernel->length = (long)bssEnd - 0x00100000;

   // add .text and .data sections from textStart...End, etc, linker script defines
   // (remember, multiboot is not portable across arch's)
   printf("___%x %x %x\n", &textEnd, &dataEnd, &bssEnd);
   objectParserCreateSection(&text, ".text", 1, 0x00100000, (long)&textEnd - 0x00100000, 2);
   objectParserCreateSection(&data, ".data", 2, (long)&textEnd, (long)&dataEnd - (long)&textEnd, 2);
   objectParserCreateSection(&bss, ".bss", 3, (long)&dataEnd, (long)&bssEnd - (long)&dataEnd, 2);

   // create all the exports
   while(symExports[i].name)
   {
      // add symbol in .text section
      printf("___creating export %s at %x\n", symExports[i].name,
             (long)symExports[i].location - 0x00100000);
      objectParserCreateSymbol(&symbol, symExports[i].name, i,
                               (long)symExports[i].location - 0x00100000);

      objectParserAddExport(objectKernel, symbol, text);

      i++;
   }

   // and then all the imports
   i = 0;
   while(symImports[i].name)
   {
      // add symbol in .text section
      printf("___creating import %s at %x\n", symImports[i].name,
             (long)symImports[i].location - 0x00100000);
      objectParserCreateSymbol(&symbol, symImports[i].name, i,
                               (long)symImports[i].location - 0x00100000);
      objectParserAddImport(objectKernel, symbol);

      objectParserCreateSymbolRelocation(&reloc, text, (long)symImports[i].location - 0x00100000,
                                         symbol, 4, False);
      objectParserAddRelocation(objectKernel, reloc);

      i++;
   }

   return ErrorNoError;
}

ErrorCode objectKernelFinal(void)
{
   printf(" --- %s %x\n", symImports[0].name, symImports[0].location);
   return ErrorNoError;
}
